/* 
    qasn1 - Asn1BerModel - Model/View Model class for qasn1 library
    Copyright(C) 2010 - Andrius da Costa Ribas

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Asn1BerModel.h"

class Asn1BerModelPrivate
{
public:
    Asn1BerModelPrivate(Asn1BerNode* node);
    Asn1BerNode* getItem(const QModelIndex &index) const;
    Asn1BerNode* m_firstNode;
};

Asn1BerModelPrivate::Asn1BerModelPrivate(Asn1BerNode* node)
{
    m_firstNode = node;
}

Asn1BerNode* Asn1BerModelPrivate::getItem(const QModelIndex &index) const
{
    if (index.isValid())
    {
        Asn1BerNode *item = static_cast<Asn1BerNode*>(index.internalPointer());
        if (item) return item;
    }
    return 0;
}

Asn1BerModel::Asn1BerModel(Asn1BerNode* node, QObject *parent) : QAbstractItemModel(parent), d( new Asn1BerModelPrivate(node) )
{
}

Asn1BerModel::~Asn1BerModel()
{
    delete d;
}

int Asn1BerModel::rowCount(const QModelIndex &parent) const
{
    int count = 0;
    
    if ( parent != QModelIndex() && !parent.isValid() )
    {
        return 0;
    }
    
    Asn1BerNode *item = d->getItem(parent);
    
    if ( item != 0 )
    {
        if ( item->isCodingPrimitive() )
        {
            return 0;
        }
        else
        {
            item = item->firstChild();
        }
    }
    else
    {
        item = d->m_firstNode;
    }
    while ( item != 0 )
    {
        count++;
        item = item->next();
    }
    return count;
}

int Asn1BerModel::columnCount(const QModelIndex & /* parent */) const
{
    return 1;
}

QVariant Asn1BerModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
    {
        return QVariant();
    }

    Asn1BerNode* item = d->getItem(index);

    if (role == Qt::DisplayRole )
    {
        QString displayText;
        
        if ( !(item->isValid()) )
        {
            return QVariant(QString("[[%1]]").arg(item->error()));
        }
        
        switch ((unsigned char) item->tagClass())
        {
            case CLASS_UNIVERSAL:
                displayText = "UNIVERSAL [";
                break;
            case CLASS_APPLICATION:
                displayText = "APPLICATION [";
                break;
            case CLASS_CONTEXT:
                displayText = "CONTEXT [";
                break;
            case CLASS_PRIVATE:
                displayText = "PRIVATE [";
                break;
            default:
                Q_ASSERT(false);
        }
        displayText+= QString::number(item->tag()) + "]";
        return QVariant(displayText);
    }
    else if ( !(item->isValid()) )
    {
        return QVariant();
    }
    else if ( role == TagRole )
    {
        return QVariant(item->tag());
    }
    else if ( role == DefinedLengthRole )
    {
        return QVariant(item->hasDefinedLength());
    }
    else if ( role == LengthRole )
    {
        if ( !(item->hasDefinedLength()) )
        {
            return QVariant();
        }
        return QVariant(item->len());
    }
    else if ( role == ValueRole )
    {
        return QVariant(item->val());
    }
    else if ( role == ClassRole )
    {
        return QVariant(item->tagClass());
    }
    else if ( role == ConstructedRole )
    {
        return QVariant(item->isCodingComplex() );
    }
    else
    {
        return QVariant();
    }
}

QVariant Asn1BerModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    // no header data
    return QVariant();
}

Qt::ItemFlags Asn1BerModel::flags(const QModelIndex &index) const
{
    if (!index.isValid())
        return 0;

    return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
}

QModelIndex Asn1BerModel::index(int row, int column, const QModelIndex &parent) const
{
    if (parent.isValid() && parent.column() != 0)
    {
        return QModelIndex();
    }
    
    if ( column != 0 )
    {
        return QModelIndex();
    }

    Asn1BerNode* parentItem = d->getItem(parent);
    Asn1BerNode* childItem;
    
    if ( parentItem == 0 )
    {
        childItem = d->m_firstNode;
    }
    else
    {
        childItem = parentItem->firstChild();
    }
    
    for ( quint64 i = 0; i < row; i++ )
    {
        if ( childItem )
        {
            childItem = childItem->next();
        }
    }
    
    if (childItem)
    {
        return createIndex(row, column, childItem);
    }
    else
    {
        return QModelIndex();
    }
}

QModelIndex Asn1BerModel::parent(const QModelIndex &index) const
{
    int count = 0;
    
    if (!index.isValid())
    {
        return QModelIndex();
    }

    Asn1BerNode* childItem = d->getItem(index);
    if ( childItem == 0 )
    {
        return QModelIndex();
    }
    Asn1BerNode* parentItem = childItem->parent();
    Asn1BerNode* iterator = parentItem;

    if (parentItem == 0)
    {
        return QModelIndex();
    }
    
    do
    {
        count++;
        iterator = iterator->prev();
    } while ( iterator != 0 );
    

    return createIndex( count - 1, 0, parentItem);
}

bool Asn1BerModel::setHeaderData ( int section, Qt::Orientation orientation, const QVariant & value, int role )
{
    // no header
    return false;
}

bool Asn1BerModel::setData ( const QModelIndex & index, const QVariant & value, int role )
{
    Asn1BerNode* node = d->getItem(index);
    
    switch ( role )
    {
        case TagRole:
            node->setTag(value.toInt());
            break;
        case DefinedLengthRole:
            if ( node->isCodingPrimitive() )
            {
                return false;
            }
            node->setDefinedLength(value.toBool());
            break;
        case ValueRole:
            node->setVal(value.toByteArray());
            break;
        case ClassRole:
            node->setClass(value.toChar().toAscii());
            break;
        case ConstructedRole:
            if ( value.toBool() )
            {
                node->setCodingComplex();
            }
            else
            {
                node->setCodingPrimitive();
            }
            break;
        default:
            return false;
    }
    emit dataChanged(index,index);
    return true;
}

bool Asn1BerModel::insertRows ( int row, int count, const QModelIndex & parent )
{
    return false;
}

bool Asn1BerModel::removeRows ( int row, int count, const QModelIndex & parent )
{
    return false;
}

#include "Asn1BerModel.moc"